//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Widget/GroupBoxes.cpp
//! @brief     Implements class GroupBoxCollapser
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Widget/GroupBoxes.h"
#include <QAction>
#include <QBoxLayout>
#include <QLabel>
#include <QMenu>
#include <QToolButton>
#include <QVariant>

//  ************************************************************************************************
//  class StaticGroupBox
//  ************************************************************************************************

StaticGroupBox::StaticGroupBox(const QString& title, QWidget* parent)
    : QWidget(parent)
{
    auto* layout = new QVBoxLayout;
    setLayout(layout);
    layout->setSpacing(0);

    auto* label = new QLabel;
    layout->setMenuBar(label);
    label->setObjectName("GroupBoxToggler");
    label->setText(title);
    label->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

    m_body = new QWidget;
    layout->addWidget(m_body);
    m_body->setObjectName("ContentArea"); // stylesheet -> transparent background
}

//  ************************************************************************************************
//  class CollapsibleGroupBox
//  ************************************************************************************************

CollapsibleGroupBox::CollapsibleGroupBox(const QString& title, QWidget* parent, bool& expanded)
    : QWidget(parent)
{
    auto* layout = new QVBoxLayout;
    setLayout(layout);
    layout->setSpacing(0);

    //... Header widget

    m_titleWidget = new QWidget;
    layout->setMenuBar(m_titleWidget);
    m_titleWidget->setObjectName("GroupBoxTitleWidget");
    m_titleWidget->setAttribute(Qt::WA_StyledBackground, true);

    m_titleLayout = new QHBoxLayout;
    m_titleWidget->setLayout(m_titleLayout);
    m_titleLayout->setSpacing(3);
    m_titleLayout->setAlignment(Qt::AlignVCenter);

    m_toggleButton = new QToolButton;
    m_titleLayout->addWidget(m_toggleButton);
    m_toggleButton->setObjectName("GroupBoxToggler");
    m_toggleButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    m_toggleButton->setCheckable(true);
    m_toggleButton->setText(title);
    m_toggleButton->setArrowType(Qt::ArrowType::DownArrow);
    m_toggleButton->setChecked(expanded);
    m_toggleButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

    //... Body

    m_body = new QWidget(this);
    layout->addWidget(m_body);
    m_body->setObjectName("ContentArea"); // stylesheet -> transparent background
    m_body->setVisible(expanded);
    m_toggleButton->setArrowType(expanded ? Qt::ArrowType::DownArrow : Qt::ArrowType::RightArrow);

    connect(m_toggleButton, &QAbstractButton::clicked,
            [tb = m_toggleButton, bo = m_body, &expanded](bool checked) {
                tb->setArrowType(checked ? Qt::ArrowType::DownArrow : Qt::ArrowType::RightArrow);
                bo->setVisible(checked);
                expanded = checked;
            });
}

void CollapsibleGroupBox::setTitle(const QString& title)
{
    m_toggleButton->setText(title);
}

void CollapsibleGroupBox::addTitleAction(QAction* action)
{
    auto* btn = new QToolButton;
    m_titleLayout->addWidget(btn);

    btn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    btn->setDefaultAction(action);
    if (action->menu() != nullptr)
        btn->setPopupMode(QToolButton::InstantPopup);

    connect(action, &QAction::changed, [=]() { btn->setVisible(action->isVisible()); });
}

void CollapsibleGroupBox::addTitleWidget(QWidget* widget)
{
    m_titleLayout->addWidget(widget);
}

void CollapsibleGroupBox::setExpanded(bool expanded)
{
    if (m_toggleButton->isChecked() != expanded)
        m_toggleButton->click();
}

//  ************************************************************************************************
//  class GroupBoxCollapser
//  ************************************************************************************************

GroupBoxCollapser* GroupBoxCollapser::installIntoGroupBox(QGroupBox* groupBox)
{
    return new GroupBoxCollapser(groupBox);
}

GroupBoxCollapser* GroupBoxCollapser::installIntoGroupBox2(QGroupBox* groupBox, bool& expanded)
{
    auto* result = new GroupBoxCollapser(groupBox);
    connect(result, &GroupBoxCollapser::toggled, [&expanded](bool b) { expanded = b; });
    return result;
}

GroupBoxCollapser* GroupBoxCollapser::findInstalledCollapser(QGroupBox* groupBox)
{
    if (groupBox == nullptr)
        return nullptr;
    return groupBox->findChild<GroupBoxCollapser*>();
}

void GroupBoxCollapser::setTitle(const QString& title)
{
    m_toggleButton->setText(title);
}

void GroupBoxCollapser::addTitleAction(QAction* action)
{
    auto* btn = new QToolButton(m_titleWidget);
    btn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    btn->setDefaultAction(action);
    if (action->menu() != nullptr)
        btn->setPopupMode(QToolButton::InstantPopup);

    m_titleLayout->addWidget(btn);

    connect(action, &QAction::changed, [=]() { btn->setVisible(action->isVisible()); });
}

void GroupBoxCollapser::addTitleWidget(QWidget* widget)
{
    m_titleLayout->addWidget(widget);
}

QWidget* GroupBoxCollapser::contentArea() const
{
    return m_contentArea;
}

GroupBoxCollapser::GroupBoxCollapser(QGroupBox* groupBox)
    : QObject(groupBox)
{
    m_toggleButton = new QToolButton(groupBox);
    m_toggleButton->setObjectName("GroupBoxToggler");
    m_toggleButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    m_toggleButton->setCheckable(true);
    m_toggleButton->setText(groupBox->title());
    m_toggleButton->setArrowType(Qt::ArrowType::DownArrow);
    m_toggleButton->setChecked(true);
    m_toggleButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

    m_titleLayout = new QHBoxLayout;
    m_titleLayout->setSpacing(3);
    m_titleLayout->setAlignment(Qt::AlignVCenter);
    m_titleLayout->addWidget(m_toggleButton);

    m_titleWidget = new QWidget(groupBox);
    m_titleWidget->setObjectName("GroupBoxTitleWidget");
    m_titleWidget->setAttribute(Qt::WA_StyledBackground, true);
    m_titleWidget->setLayout(m_titleLayout);

    m_contentArea = new QWidget(groupBox);
    m_contentArea->setObjectName("ContentArea");
    m_contentArea->setLayout(groupBox->layout());

    auto* mainLayout = new QVBoxLayout;
    mainLayout->setSpacing(0);
    mainLayout->setMenuBar(m_titleWidget);
    mainLayout->addWidget(m_contentArea);

    groupBox->setTitle("");          // title has been read above
    groupBox->setLayout(mainLayout); // overwrites layout that has been read above

    connect(m_toggleButton, &QAbstractButton::clicked, this, &GroupBoxCollapser::toggle,
            Qt::UniqueConnection);
}

void GroupBoxCollapser::toggle(bool checked)
{
    m_toggleButton->setArrowType(checked ? Qt::ArrowType::DownArrow : Qt::ArrowType::RightArrow);

    if (m_toggleButton->isChecked())
        m_contentArea->show();
    else
        m_contentArea->hide();
    emit toggled(m_toggleButton->isChecked());
}
